@@ -49,8 +49,6 @@ require_version('Gdk', '3.0')
 from gi.repository import Gdk
 require_version('Gtk', '3.0')
 from gi.repository import Gtk
-require_version('IBus', '1.0')
-from gi.repository import IBus
 from gi.repository import GObject
 import itb_emoji
 import version
@@ -89,7 +87,7 @@ def parse_args():
         default=None,
         help=('Set a font to display emoji. '
               + 'If not specified, the font is read from the config file. '
-              + 'To use the system default font specify "". '
+              + 'To use the system default font specify "emoji". '
               + 'default: "%(default)s"'))
     parser.add_argument(
         '-s', '--fontsize',
@@ -117,17 +115,50 @@ def parse_args():
               + 'Slows the search down and is usually not needed. '
               + 'default: %(default)s'))
     parser.add_argument(
-        '--use_vs16',
+        '--fallback',
+        nargs='?',
+        type=int,
+        action='store',
+        default=None,
+        help=('Whether to use fallback fonts when rendering emoji. '
+              + 'If not 0, pango will use fallback fonts as necessary. '
+              + 'If 0, pango will use only glyphs from '
+              + 'the closest matching font on the system. No fallback '
+              + 'will be done to other fonts on the system that '
+              + 'might contain the glyhps needed to render an emoji. '
+              + 'default: "%(default)s"')
+        )
+    parser.add_argument(
+        '--emoji_unicode_min',
+        nargs='?',
+        type=str,
+        action='store',
+        default='1.0',
+        help=('Load only emoji which were added to Unicode '
+              + 'not earlier than this Unicode version. '
+              + 'default: %(default)s'))
+    parser.add_argument(
+        '--emoji_unicode_max',
+        nargs='?',
+        type=str,
+        action='store',
+        default='100.0',
+        help=('Load only emoji which were added to Unicode '
+              + 'not later than this Unicode version. '
+              + 'default: %(default)s'))
+    parser.add_argument(
+        '--non_fully_qualified',
         action='store_true',
         default=False,
-        help=('Use the emoji variation selector '
+        help=('Do not use the emoji variation selector '
               + '(U+FE0F VARIATION SELECTOR-16) '
-              + 'in emoji sequences following the recommendations '
-              + 'of unicode.org. Although doing this is '
-              + '“strongly recommended for keyboards” by unicode.org '
-              + 'for “good fallback behavior on older systems” '
-              + 'it seems to have only disadvantages with the currently '
-              + 'available fonts. '
+              + 'in emoji sequences. Without this option, U+FE0F is used '
+              + 'according to the recommendation of unicode.org. '
+              + 'On some systems, and with some fonts this might make '
+              + 'some emoji sequences render incorrectly though. '
+              + 'If you see some emoji sequences rendered as several glyphs '
+              + 'instead of being rendered as a single glyph, you can try '
+              + 'whether this option improves the rendering for you. '
               + 'default: %(default)s'))
     parser.add_argument(
         '-d', '--debug',
@@ -153,9 +184,12 @@ class EmojiPickerUI(Gtk.Window):
                  languages=('en_US',),
                  modal=False,
                  unicode_data_all=False,
-                 use_variation_selector_16=False,
+                 emoji_unicode_min=1.0,
+                 emoji_unicode_max=100.0,
+                 non_fully_qualified=False,
                  font=None,
-                 fontsize=None):
+                 fontsize=None,
+                 fallback=None):
         Gtk.Window.__init__(self, title='🚀 ' + _('Emoji Picker'))
 
         self.set_name('EmojiPicker')
@@ -199,17 +233,11 @@ class EmojiPickerUI(Gtk.Window):
         self._modal = modal
         self.set_modal(self._modal)
         available_fonts = self._list_font_names()
-        default_fonts = [
-            'Noto Color Emoji',
-            'Emoji One',
-            'Symbola',
-        ]
-        self._font = 'Symbola'
-        for default_font in default_fonts:
-            if default_font in available_fonts:
-                self._font = default_font
-                break
+        self._font = 'emoji'
+        if available_fonts:
+            self._font = available_fonts[0]
         self._fontsize = 24
+        self._fallback = True
         self._font_popover = None
         self._font_popover_scroll = None
         self._font_popover_listbox = None
@@ -221,15 +249,23 @@ class EmojiPickerUI(Gtk.Window):
             self._font = font
         if not fontsize is None:
             self._fontsize = fontsize
+        if not fallback is None:
+            self._fallback = bool(fallback)
         self._save_options()
         self.connect('destroy-event', self.on_destroy_event)
         self.connect('delete-event', self.on_delete_event)
         self.connect('key-press-event', self.on_main_window_key_press_event)
         self._languages = languages
+        self._emoji_unicode_min = emoji_unicode_min
+        self._emoji_unicode_max = emoji_unicode_max
+        self._unicode_data_all = unicode_data_all
+        self._non_fully_qualified = non_fully_qualified
         self._emoji_matcher = itb_emoji.EmojiMatcher(
             languages=self._languages,
-            unicode_data_all=unicode_data_all,
-            use_variation_selector_16=use_variation_selector_16)
+            unicode_data_all=self._unicode_data_all,
+            emoji_unicode_min=self._emoji_unicode_min,
+            emoji_unicode_max=self._emoji_unicode_max,
+            non_fully_qualified=self._non_fully_qualified)
         self._gettext_translations = {}
         for language in itb_emoji.expand_languages(self._languages):
             mo_file = gettext.find(DOMAINNAME, languages=[language])
@@ -292,6 +328,8 @@ class EmojiPickerUI(Gtk.Window):
             'clicked', self.on_main_menu_button_clicked)
         self._toggle_search_button = Gtk.Button.new_from_icon_name(
             'edit-find-symbolic', Gtk.IconSize.BUTTON)
+        self._toggle_search_button.set_tooltip_text(
+            _('Search for emoji'))
         self._toggle_search_button.connect(
             'clicked', self.on_toggle_search_button_clicked)
         self._header_bar.pack_start(self._toggle_search_button)
@@ -302,12 +340,16 @@ class EmojiPickerUI(Gtk.Window):
             Gtk.Image.new_from_icon_name(
                 'preferences-desktop-font', Gtk.IconSize.BUTTON))
         self._font_button.set_label(self._font)
+        self._font_button.set_tooltip_text(
+            _('Set the font to display emoji'))
         self._font_button.connect(
             'clicked', self.on_font_button_clicked)
         self._header_bar.pack_start(self._font_button)
         self._fontsize_spin_button = Gtk.SpinButton()
         self._fontsize_spin_button.set_numeric(True)
         self._fontsize_spin_button.set_can_focus(True)
+        self._fontsize_spin_button.set_tooltip_text(
+            _('Set font size'))
         self._fontsize_spin_button.connect(
             'grab-focus', self.on_fontsize_spin_button_grab_focus)
         self._fontsize_adjustment = Gtk.Adjustment()
@@ -319,6 +361,24 @@ class EmojiPickerUI(Gtk.Window):
         self._fontsize_adjustment.connect(
             'value-changed', self.on_fontsize_adjustment_value_changed)
         self._header_bar.pack_start(self._fontsize_spin_button)
+        self._fallback_check_button = Gtk.CheckButton()
+        # Translators: This is a checkbox which enables or disables
+        # the pango font fallback. If font fallback is off, only the
+        # font selected in the font menu is used to display emoji
+        # (if possible, this may not always work, sometimes other fonts
+        # may be used even if font fallback is off).
+        # If font fallback is on, other fonts will be tried for emoji
+        # or parts of emoji sequences which cannot be displayed in the
+        # selected font.
+        self._fallback_check_button.set_label(_('Fallback'))
+        self._fallback_check_button.set_tooltip_text(
+            _('Whether to use font fallback for emoji or parts of emoji '
+              + 'sequences which cannot be displayed using the '
+              + 'selected font.'))
+        self._fallback_check_button.set_active(self._fallback)
+        self._fallback_check_button.connect(
+            'toggled', self.on_fallback_check_button_toggled)
+        self._header_bar.pack_start(self._fallback_check_button)
         self._spinner = Gtk.Spinner()
         self._header_bar.pack_end(self._spinner)
         self.set_titlebar(self._header_bar)
@@ -566,7 +626,8 @@ class EmojiPickerUI(Gtk.Window):
                     '\n' + self._translate_key('Keywords', language) + ': '
                     + html.escape(', '.join(keywords)))
                 description_empty = False
-            categories = self._emoji_matcher.categories(emoji, language=language)
+            categories = self._emoji_matcher.categories(
+                emoji, language=language)
             if categories:
                 description += (
                     '\n' + self._translate_key('Categories', language) + ': '
@@ -576,12 +637,15 @@ class EmojiPickerUI(Gtk.Window):
                 descriptions.append(description)
         fonts_description = _('Fonts used to render this emoji:')
         for text, font_family in itb_pango.get_fonts_used_for_text(
-            self._font + ' ' + str(self._fontsize), emoji):
+                self._font + ' ' + str(self._fontsize), emoji,
+                fallback=self._fallback):
             fonts_description += '\n'
             for char in text:
                 fonts_description += (
-                    '<span font_desc="%s">' %self._font + char + '</span>'
-                    + ' U+%X' %ord(char))
+                    '<span font="%s" fallback="%s" >'
+                    %(self._font, str(self._fallback).lower())
+                    + char + '</span>'
+                    + ' <span fallback="false">U+%X</span>' %ord(char))
             fonts_description += ': ' + font_family
         descriptions.append(fonts_description)
         if self._emoji_matcher.unicode_version(emoji):
@@ -610,7 +674,8 @@ class EmojiPickerUI(Gtk.Window):
         '''
         tooltip_text = _('Left click to copy') + '\n'
         if len(self._emoji_matcher.skin_tone_variants(emoji)) > 1:
-            tooltip_text += _('Long press or middle click for skin tones')  + '\n'
+            tooltip_text += (
+                _('Long press or middle click for skin tones')  + '\n')
         tooltip_text += _('Right click for info')
         label.set_tooltip_text(tooltip_text)
 
@@ -637,8 +702,8 @@ class EmojiPickerUI(Gtk.Window):
         self._flowbox.connect('child-activated', self.on_emoji_selected)
         for long_press_gesture in self._long_press_gestures:
             # disconnecting is necessary to avoid warnings to stdout:
-            for id in long_press_gesture[1]:
-                long_press_gesture[0].disconnect(id)
+            for ids in long_press_gesture[1]:
+                long_press_gesture[0].disconnect(ids)
         self._long_press_gestures = []
 
     def _fill_flowbox_browse(self):
@@ -677,15 +742,6 @@ class EmojiPickerUI(Gtk.Window):
             while Gtk.events_pending():
                 Gtk.main_iteration()
             if not is_recently_used:
-                if len(emoji) > 1:
-                    has_skin_tone_modifier = False
-                    for modifier in itb_emoji.SKIN_TONE_MODIFIERS:
-                        if modifier in emoji:
-                            has_skin_tone_modifier = True
-                    if has_skin_tone_modifier:
-                        # Skip all emoji which already contain a
-                        # skin tone modifier:.
-                        continue
                 skin_tone_variants = self._emoji_matcher.skin_tone_variants(
                     emoji)
                 if len(skin_tone_variants) > 1:
@@ -704,13 +760,13 @@ class EmojiPickerUI(Gtk.Window):
             label = Gtk.Label()
             # Make font for emoji large using pango markup
             text = (
-                '<span font="%s %s">'
-                %(self._font, self._fontsize)
+                '<span font="%s %s" fallback="%s">'
+                %(self._font, self._fontsize, str(self._fallback).lower())
                 + html.escape(emoji)
                 + '</span>')
             if itb_emoji.is_invisible(emoji):
                 text += (
-                    '<span font="%s">'
+                    '<span fallback="false" font="%s">'
                     %(self._fontsize / 2)
                     + ' U+%X ' %ord(emoji)
                     + self._emoji_matcher.name(emoji)
@@ -750,13 +806,17 @@ class EmojiPickerUI(Gtk.Window):
             long_press_gesture.set_propagation_phase(
                 Gtk.PropagationPhase.CAPTURE)
             id_pressed = long_press_gesture.connect(
-                'pressed', self.on_flowbox_event_box_long_press_pressed, event_box)
+                'pressed',
+                self.on_flowbox_event_box_long_press_pressed, event_box)
             id_begin = long_press_gesture.connect(
-                'begin', self.on_flowbox_event_box_long_press_begin)
+                'begin',
+                self.on_flowbox_event_box_long_press_begin)
             id_cancel = long_press_gesture.connect(
-                'cancel', self.on_flowbox_event_box_long_press_cancel)
+                'cancel',
+                self.on_flowbox_event_box_long_press_cancel)
             id_cancelled = long_press_gesture.connect(
-                'cancelled', self.on_flowbox_event_box_long_press_cancelled)
+                'cancelled',
+                self.on_flowbox_event_box_long_press_cancelled)
             self._long_press_gestures.append(
                 (long_press_gesture,
                  (id_pressed,
@@ -803,10 +863,15 @@ class EmojiPickerUI(Gtk.Window):
         if ('font' in options_dict
                 and isinstance(options_dict['font'], str)):
             self._font = options_dict['font']
+            if self._font == '':
+                self._font = 'emoji'
         if ('fontsize' in options_dict
                 and (isinstance(options_dict['fontsize'], int)
                      or isinstance(options_dict['fontsize'], float))):
             self._fontsize = options_dict['fontsize']
+        if ('fallback' in options_dict
+                and (isinstance(options_dict['fallback'], bool))):
+            self._fallback = options_dict['fallback']
 
     def _save_options(self):
         '''
@@ -815,6 +880,7 @@ class EmojiPickerUI(Gtk.Window):
         options_dict = {
             'font': self._font,
             'fontsize': self._fontsize,
+            'fallback': self._fallback,
             }
         with open(self._options_file,
                   mode='w',
@@ -835,9 +901,10 @@ class EmojiPickerUI(Gtk.Window):
         '''
         Read the recently use emoji from a file
         '''
+        recently_used_emoji = {}
         if os.path.isfile(self._recently_used_emoji_file):
             try:
-                self._recently_used_emoji = eval(open(
+                recently_used_emoji = eval(open(
                     self._recently_used_emoji_file,
                     mode='r',
                     encoding='UTF-8').read())
@@ -853,13 +920,21 @@ class EmojiPickerUI(Gtk.Window):
                         'File %s has been read and evaluated.\n'
                         %self._recently_used_emoji_file)
             finally: # executes always
-                if not isinstance(self._recently_used_emoji, dict):
+                if not isinstance(recently_used_emoji, dict):
                     if _ARGS.debug:
                         sys.stdout.write(
                             'Not a dict: '
-                            + 'repr(self._recently_used_emoji) = %s\n'
-                            %self._recently_used_emoji)
-                    self._recently_used_emoji = {}
+                            + 'repr(recently_used_emoji) = %s\n'
+                            %recently_used_emoji)
+                    recently_used_emoji = {}
+        self._recently_used_emoji = {}
+        for emoji, value in recently_used_emoji.items():
+            # add or remove vs16 according to the current setting:
+            if emoji:
+                self._recently_used_emoji[
+                    self._emoji_matcher.variation_selector_16_normalize(
+                        emoji,
+                        non_fully_qualified=self._non_fully_qualified)] = value
         if not self._recently_used_emoji:
             self._init_recently_used()
         self._cleanup_recently_used()
@@ -950,8 +1025,8 @@ class EmojiPickerUI(Gtk.Window):
         '''
         Print some profiling information to stdout.
         '''
-        profile.disable()
-        stats = pstats.Stats(profile)
+        PROFILE.disable()
+        stats = pstats.Stats(PROFILE)
         stats.strip_dirs()
         stats.sort_stats('cumulative')
         stats.print_stats('emoji_picker', 25)
@@ -1151,8 +1226,8 @@ class EmojiPickerUI(Gtk.Window):
             label = Gtk.Label()
             # Make font for emoji large using pango markup
             label.set_text(
-                '<span font="%s %s">'
-                %(self._font, self._fontsize)
+                '<span font="%s %s" fallback="%s">'
+                %(self._font, self._fontsize, str(self._fallback).lower())
                 + html.escape(emoji)
                 + '</span>'
                 + '<span font="%s">'
@@ -1192,13 +1267,17 @@ class EmojiPickerUI(Gtk.Window):
             long_press_gesture.set_propagation_phase(
                 Gtk.PropagationPhase.CAPTURE)
             id_pressed = long_press_gesture.connect(
-                'pressed', self.on_flowbox_event_box_long_press_pressed, event_box)
+                'pressed',
+                self.on_flowbox_event_box_long_press_pressed, event_box)
             id_begin = long_press_gesture.connect(
-                'begin', self.on_flowbox_event_box_long_press_begin)
+                'begin',
+                self.on_flowbox_event_box_long_press_begin)
             id_cancel = long_press_gesture.connect(
-                'cancel', self.on_flowbox_event_box_long_press_cancel)
+                'cancel',
+                self.on_flowbox_event_box_long_press_cancel)
             id_cancelled = long_press_gesture.connect(
-                'cancelled', self.on_flowbox_event_box_long_press_cancelled)
+                'cancelled',
+                self.on_flowbox_event_box_long_press_cancelled)
             self._long_press_gestures.append(
                 (long_press_gesture,
                  (id_pressed,
@@ -1342,7 +1421,7 @@ class EmojiPickerUI(Gtk.Window):
             + r'(<span[^<]*?>(?P<name>[^<]+?)</span>)?')
         match = pattern.match(text)
         if match:
-            emoji  = html.unescape(match.group('emoji'))
+            emoji = html.unescape(match.group('emoji'))
         if match.group('name'):
             name = html.unescape(match.group('name'))
         if _ARGS.debug:
@@ -1364,6 +1443,17 @@ class EmojiPickerUI(Gtk.Window):
         return False
 
     def _emoji_event_box_selected(self, event_box):
+        '''
+        Called when an event box containing an emoji
+        was selected in the flowbox.
+
+        The emoji is then copied to the clipboard and a popover
+        pops up for a short time to notify the user that the emoji
+        has been copied to the clipboard.
+
+        :param event_box: The event box which contains the emoji
+        :type event_box: Gtk.EventBox object
+        '''
         # Use .get_label() instead of .get_text() to fetch the text
         # from the label widget including any embedded underlines
         # indicating mnemonics and Pango markup. The emoji is in
@@ -1450,13 +1540,13 @@ class EmojiPickerUI(Gtk.Window):
             (emoji, name) = self._parse_emoji_and_name_from_text(text)
             if emoji:
                 new_text = (
-                    '<span font="%s %s">'
-                    %(self._font, self._fontsize)
+                    '<span font="%s %s" fallback="%s">'
+                    %(self._font, self._fontsize, str(self._fallback).lower())
                     + html.escape(emoji)
                     + '</span>')
                 if name:
                     new_text += (
-                        '<span font="%s">'
+                        '<span fallback="false" font="%s">'
                         %(self._fontsize / 2)
                         + html.escape(name)
                         + '</span>')
@@ -1499,7 +1589,7 @@ class EmojiPickerUI(Gtk.Window):
         text = flowbox_child.get_child().get_label()
         if _ARGS.debug:
             sys.stdout.write('on_skin_tone_selected() text = %s\n' %text)
-        (emoji, name) = self._parse_emoji_and_name_from_text(text)
+        (emoji, dummy_name) = self._parse_emoji_and_name_from_text(text)
         if not emoji:
             return
         self._set_clipboards(emoji)
@@ -1525,8 +1615,8 @@ class EmojiPickerUI(Gtk.Window):
             # in a search results flowbox and we do *not* want
             # to replace the emoji.
             new_text = (
-                '<span font="%s %s">'
-                %(self._font, self._fontsize)
+                '<span font="%s %s" fallback="%s">'
+                %(self._font, self._fontsize, str(self._fallback).lower())
                 + html.escape(emoji)
                 + '</span>')
             label.set_text(new_text)
@@ -1574,7 +1664,8 @@ class EmojiPickerUI(Gtk.Window):
                 'on_flowbox_event_box_long_press_cancelled() %s\n'
                 %repr(gesture))
 
-    def on_flowbox_event_box_long_press_pressed(self, gesture, x, y, event_box):
+    def on_flowbox_event_box_long_press_pressed(
+            self, gesture, x, y, event_box):
         '''
         :param gesture: The object which received the signal
         :type gesture: Gtk.GestureLongPress object
@@ -1691,8 +1782,8 @@ class EmojiPickerUI(Gtk.Window):
         for skin_tone_variant in skin_tone_variants:
             label = Gtk.Label()
             label.set_text(
-                '<span font="%s %s">'
-                %(self._font, self._fontsize)
+                '<span font="%s %s" fallback="%s">'
+                %(self._font, self._fontsize, str(self._fallback).lower())
                 + html.escape(skin_tone_variant)
                 + '</span>')
             label.set_use_markup(True)
@@ -1799,8 +1890,8 @@ class EmojiPickerUI(Gtk.Window):
         label.set_vexpand(False)
         label.set_halign(Gtk.Align.START)
         label.set_markup(
-            '<span font_desc="%s %s">'
-            %(self._font, self._fontsize * 3)
+            '<span font="%s %s" fallback="%s">'
+            %(self._font, self._fontsize * 3, str(self._fallback).lower())
             + html.escape(emoji)
             + '</span>')
         emoji_info_popover_listbox.insert(label, -1)
@@ -1811,6 +1902,7 @@ class EmojiPickerUI(Gtk.Window):
             label_description.set_margin_end(margin)
             label_description.set_margin_top(margin)
             label_description.set_margin_bottom(margin)
+            label_description.set_selectable(True)
             label_description.set_hexpand(False)
             label_description.set_vexpand(False)
             label_description.set_halign(Gtk.Align.START)
@@ -1818,7 +1910,7 @@ class EmojiPickerUI(Gtk.Window):
             label_description.set_markup(description)
             emoji_info_popover_listbox.insert(label_description, -1)
         if (self._emoji_matcher.cldr_order(emoji) < 0xFFFFFFFF
-            or self._emoji_matcher.properties(emoji)):
+                or self._emoji_matcher.properties(emoji)):
             linkbutton = Gtk.LinkButton.new_with_label(
                 _('Lookup on emojipedia'))
             linkbutton.set_uri(
@@ -1909,15 +2001,51 @@ class EmojiPickerUI(Gtk.Window):
         self._busy_start()
         GLib.idle_add(self._change_flowbox_font)
 
+    def on_fallback_check_button_toggled(self, check_button):
+        '''
+        The fallback check button in the header bar has been toggled
+
+        :param toggle_button: The check button used to select whether
+                              fallback fonts should be used.
+        :type adjustment: Gtk.CheckButton object
+        '''
+        self._fallback = check_button.get_active()
+        if _ARGS.debug:
+            sys.stdout.write(
+                'on_fallback_check_button_toggled() self._fallback = %s\n'
+                %self._fallback)
+        self._save_options()
+        self._busy_start()
+        GLib.idle_add(self._change_flowbox_font)
+
     def _list_font_names(self):
         '''
         Returns a list of font names available on the system
 
         :rtype: List of strings
         '''
+        good_emoji_fonts = [
+            'Noto Color Emoji 🎨',
+            'Apple Color Emoji 🎨',
+            'Emoji Two 🎨', # color
+            'Emoji One 🎨', # color
+            'Symbola 🙾', # black and white
+            'Noto Emoji 🙾', # black and white
+            'Android Emoji 🙾', # black and white
+            'Segoe UI Emoji 🙾', # seems to be black and white
+            'Twitter Color Emoji 🙾', # seems to be black and white
+        ]
+        available_good_emoji_fonts = ['emoji (' + _('System default') + ')']
         pango_context = self.get_pango_context()
         families = pango_context.list_families()
-        names = [family.get_name() for family in families]
+        names = sorted([family.get_name() for family in families])
+        for font in good_emoji_fonts:
+            name = font.split(' ')[0]
+            if name in names:
+                names.remove(name)
+                available_good_emoji_fonts.append(font)
+        available_good_emoji_fonts.append('')
+        names = available_good_emoji_fonts + names
         return names
 
     def _fill_listbox_font(self, filter_text):
@@ -1945,7 +2073,7 @@ class EmojiPickerUI(Gtk.Window):
         self._font_popover_listbox.set_activate_on_single_click(True)
         self._font_popover_listbox.connect(
             'row-selected', self.on_font_selected)
-        fonts = [''] + [
+        fonts = [
             font
             for font in self._list_font_names()
             if filter_text.replace(' ', '').lower()
@@ -1986,10 +2114,18 @@ class EmojiPickerUI(Gtk.Window):
         :param listbox_row: A row containing a font name
         :type listbox_row: Gtk.ListBoxRow object
         '''
-        font = listbox_row.get_child().get_text()
+        font = listbox_row.get_child().get_text().split(' ')[0]
         if _ARGS.debug:
             sys.stdout.write(
                 'on_font_selected() font = %s\n' %repr(font))
+        if font == '':
+            font = 'emoji'
+        if font != self._font and font == 'emoji':
+            self._fallback = True
+            self._fallback_check_button.set_active(True)
+        else:
+            self._fallback = False
+            self._fallback_check_button.set_active(False)
         if GTK_VERSION >= (3, 22, 0):
             self._font_popover.popdown()
         self._font_popover.hide()
@@ -2073,8 +2209,8 @@ if __name__ == '__main__':
         sys.stdout.write('--- %s ---\n' %time.strftime('%Y-%m-%d: %H:%M:%S'))
         import cProfile
         import pstats
-        profile = cProfile.Profile()
-        profile.enable()
+        PROFILE = cProfile.Profile()
+        PROFILE.enable()
 
     # Workaround for
     # https://bugzilla.gnome.org/show_bug.cgi?id=622084
@@ -2098,7 +2234,10 @@ if __name__ == '__main__':
         languages=get_languages(),
         font=_ARGS.font,
         fontsize=_ARGS.fontsize,
+        fallback=_ARGS.fallback,
         modal=_ARGS.modal,
         unicode_data_all=_ARGS.all,
-        use_variation_selector_16=_ARGS.use_vs16)
+        emoji_unicode_min=_ARGS.emoji_unicode_min,
+        emoji_unicode_max=_ARGS.emoji_unicode_max,
+        non_fully_qualified=_ARGS.non_fully_qualified)
     Gtk.main()
